//------------------------------------------------------------------------------------------------------------------------------------------------
// emonGLCD Home Energy Monitor Debugger


// Receives RF data & displays it in various formats

// If Enter switch is held pressed during power-on, 868 MHz is used, otherwise 433 MHz

// When running: 
//    'Enter' switch moves cursor to Byte 0, 
//    'Up' switch moves cursor 1 byte higher (right),  
//    'Down' switch moves cursor 1 byte lower (left).

//   "Node" shows the ID of the last message received.
//   "Count" shows the number of bytes in the last message received.
//   The line immediately below Node, frequency & Count shows time since last message was received
//    - scale approx 3 mm/s, or 20 s for the full display width.
//   Byte values are shown in Hex. The first 12 are displayed. As the cursor is moved to the right, the bytes 
//   displayed scroll left. 
//   The bytes underlined are interpreted as a char value, a signed integer value and an unsigned long value.
//   Their position in the message is a shown (Zero-based).

//   The Green LEDs flash each time a message is received. The Red LEDs light approx 20 s after the last message.
 

// emonGLCD documentation http://openEnergyMonitor.org/emon/emonglcd

// GLCD library by Jean-Claude Wippler: JeeLabs.org
// 2010-05-28 <jcw@equi4.com> http://opensource.org/licenses/mit-license.php
//
// Authors: Glyn Hudson and Trystan Lea
// Part of the: openenergymonitor.org project
// Licenced under GNU GPL V3
// http://openenergymonitor.org/emon/license

// THIS SKETCH REQUIRES:

// Libraries in the standard arduino libraries folder:
//
//	- JeeLib		https://github.com/jcw/jeelib
//	- GLCD_ST7565		https://github.com/jcw/glcdlib
//
//
//-------------------------------------------------------------------------------------------------------------------------------------------------

#include <JeeLib.h>
#include <GLCD_ST7565.h>
#include <avr/pgmspace.h>
GLCD_ST7565 glcd;
#include "utility/font_clR6x8.h"    //fonts
#include "utility/font_clR4x6.h"


//--------------------------------------------------------------------------------------------
// RFM12B Settings
//--------------------------------------------------------------------------------------------
#define MYNODE 20            // Should be unique on network, node ID 30 reserved for base station
#define group 210 


unsigned long fast_update, slow_update;
unsigned long next_rf;                             //time when next rf message should have been received.  

#define LEDON HIGH
#define LEDOFF LOW
#define TIMEOUT 20000L       		// The red LED lights this many milliseconds after the last message received

// #define DEBUG

const int greenLED=6;        		// Green tri-color LED
const int redLED=9;          		// Red tri-color LED
const int LDRpin=4;    		 		// analog pin of onboard lightsensor 
const int enterswitchpin=15;        // digital pin of enter switch - low when pressed
const int upswitchpin=16;           // digital pin of up switch - low when pressed
const int downswitchpin=19;         // digital pin of down switch - low when pressed
int freq;

boolean last_enter_switch_state, enter_switch_state, 
	last_up_switch_state, up_switch_state, last_down_switch_state, down_switch_state;

int bytenum;
int tx_nodeID;
int tx_count;
char tx_data[80];

//-------------------------------------------------------------------------------------------- 
// Flow control
//-------------------------------------------------------------------------------------------- 
unsigned long last_tx;          	// Used to count time from last update
unsigned long now;


//--------------------------------------------------------------------------------------------
// Setup
//--------------------------------------------------------------------------------------------
void setup()
{
  delay(500); 				   //wait for power to settle before firing up the RF
  
#ifdef DEBUG
  Serial.begin(9600);
  Serial.println("emonGLCD Debugger");
  Serial.println("openenergymonitor.org");
#endif
  if (digitalRead(enterswitchpin) == 1)
	  freq = RF12_868MHZ;
  else
	  freq = RF12_433MHZ;

  rf12_initialize(MYNODE, freq, group);
		
  delay(100);				   //wait for RF to settle before turning on display
  glcd.begin(0x19);
  glcd.backLight(200);
  glcd.setFont(font_clR6x8);
  
  pinMode(greenLED, OUTPUT); 
  pinMode(redLED, OUTPUT); 
  draw_page(tx_nodeID, tx_data, tx_count, bytenum);

}

//--------------------------------------------------------------------------------------------
// Loop
//--------------------------------------------------------------------------------------------
void loop()
{
  static bool update = true;
  now = millis();
  
  if (rf12_recvDone())
  {
    if (rf12_crc == 0 && (rf12_hdr & RF12_HDR_CTL) == 0)  // and no rf errors
    {
      int node_id = (rf12_hdr & 0x1F);

	  digitalWrite(redLED, LEDOFF); 
      digitalWrite(greenLED, LEDON);		// turn green LED on to indicate RF receive 
      tx_nodeID=rf12_hdr & 0x1F;        	// extract node ID from received packet
      tx_count = rf12_len;              	// get data structure size
      delay(8);
	  digitalWrite(greenLED,LEDOFF);        // Turn green LED OFF

	  for (byte i=0; i<tx_count && i < 80; i++)
          tx_data[i] = (unsigned char)rf12_data[i];

		  
	  #ifdef DEBUG 
      Serial.print("RF received - Node ");Serial.println(tx_nodeID);

      for (byte i=0; i<tx_count; i+=2)
      {
          int num = ((unsigned char)rf12_data[i+1] << 8 | (unsigned char)rf12_data[i]);
		  if (i) Serial.print(", ");
          Serial.print("Val");Serial.print(i/2+1);Serial.print("=");Serial.print(num);
      }
	  Serial.println(" ");
	  #endif
	  	  
      next_rf = now + TIMEOUT;
	  update = true;
    } 
  }
  if (now > next_rf)
	  digitalWrite(redLED,LEDON); 

  //--------------------------------------------------------------------------------------------
  // Display update every 200 ms - this also defines switch response time
  //--------------------------------------------------------------------------------------------
  if ((now-fast_update)>200)
  {
	fast_update = now;
    
    last_enter_switch_state = enter_switch_state;
    enter_switch_state = digitalRead(enterswitchpin);  
	last_up_switch_state = up_switch_state;
    up_switch_state = digitalRead(upswitchpin);  
	last_down_switch_state = down_switch_state;
    down_switch_state = digitalRead(downswitchpin);  
    if (!last_enter_switch_state && enter_switch_state) 
	{
		update = true;
		bytenum = 0;
	}
    if (!last_up_switch_state && up_switch_state) 
	{
		update = true;
		bytenum++;
	}
    if (!last_down_switch_state && down_switch_state) 
	{
		update = true;
		bytenum--;
	}
	if (bytenum < 0)
		bytenum = 0;
	if (bytenum > tx_count)
		bytenum = tx_count;

	if (update)
		draw_page(tx_nodeID, tx_data, tx_count, bytenum);
	update = false;
	
	draw_timebar();
  } 
}

void draw_timebar(void)
{
	if ((next_rf - now) < TIMEOUT)
	{
		int end = (next_rf - now) / (TIMEOUT/120);
		glcd.drawLine(end, 7, 120, 7, 0);	// 120 is the max line length. N.B. Drawing a white line to erase!
		glcd.refresh();
	}
}


void draw_page(unsigned int node, char* data, unsigned int dcount, unsigned int hilight)
{ 
	char str[20];
	glcd.clear();
	glcd.fillRect(0,0,128,64,0);
	glcd.setFont(font_clR4x6);
	glcd.drawString_P(0,  0, PSTR("Node: "));
	itoa((int)node,str,10);
	glcd.drawString(30, 0, str);
	if (freq == RF12_868MHZ)
		glcd.drawString_P(45,  0, PSTR("868 MHz"));
	else if (freq == RF12_433MHZ)
		glcd.drawString_P(45,  0, PSTR("433 MHz"));
	glcd.drawString_P(80,  0, PSTR("Count: "));
	itoa((int)tx_count,str,10);
	glcd.drawString(110, 0, str);
	
	
	
	char dbyte=hilight<4 ? 0 : hilight-4;
	for (unsigned int i=0; i<12; i++, dbyte++)
	{
		if (i < dcount)
		{
			itoa((int)(unsigned char)data[dbyte],str,16);
			if (str[1] == '\0')
			{	
				str[1] = str[0]; 
				str[0] = '0';
			}
			glcd.drawString(i*10, 10, str);
		}
	}
	char hilightPos = hilight < 4 ? hilight*10 : 40;
	glcd.drawLine(hilightPos, 16, hilightPos +6,  16, 1);
	glcd.drawLine(hilightPos, 17, hilightPos +16, 17, 1);
	glcd.drawLine(hilightPos, 18, hilightPos +36, 18, 1);
	
	glcd.drawString_P(0,  38, PSTR("char: "));
	int num = ((unsigned char)data[hilight]);
	itoa(num, str, 10);
	glcd.drawString(30, 38, str);
	glcd.drawString_P(83,  38, PSTR("Byte: "));
	itoa((int)hilight,str,10);
	glcd.drawString(106, 38, str);
	
	glcd.drawString_P(0,  48, PSTR("int: "));
	num = ((unsigned char)data[hilight+1] << 8 | (unsigned char)data[hilight]);
	itoa(num, str, 10);
	glcd.drawString(30, 48, str);
	glcd.drawString_P(83,  48, PSTR("Bytes: "));
	itoa((int)hilight,str,10);	
	glcd.drawString(106, 48, str);
	glcd.drawString_P(114,  48, PSTR("-"));
	itoa((int)hilight+1,str,10);	
	glcd.drawString(118, 48, str);
		
	glcd.drawString_P(0,  58, PSTR("ulong: "));
	unsigned long lnum = ((unsigned char)data[hilight+3] * 16777216UL) | ((unsigned char)data[hilight+2] * 65536UL) | ((unsigned char)data[hilight+1] * 256UL) | (unsigned char)data[hilight];
	ultoa(lnum, str, 10);
	glcd.drawString(30, 58, str);
	glcd.drawString_P(83,  58, PSTR("Bytes: "));
	itoa((int)hilight,str,10);	
	glcd.drawString(106, 58, str);
	glcd.drawString_P(114,  58, PSTR("-"));
	itoa((int)hilight+3,str,10);	
	glcd.drawString(118, 58, str);

	glcd.drawLine(120, 7, 0, 7, 1);	// 120 is the max line length.
	
    glcd.refresh();
}
